# -*- mode: python; coding: utf-8 -*-
#
# Copyright (C) 2015 <contact@redhat.com>
#
# Author: Loic Dachary <loic@dachary.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
#
import argparse
from ceph_workbench import util
from ceph_workbench import wbgit
from ceph_workbench import wbgithub
from ceph_workbench import wbredmine
import logging
import re
import subprocess


class WBBackportSetRelease(object):

    def __init__(self, args):
        self.args = args
        self.git = wbgit.WBRepo(args)
        self.github = wbgithub.WBGitHub(args)
        self.redmine = wbredmine.WBRedmine(args)

    def open(self):
        self.git.open().index()
        self.github.open().index()
        self.redmine.open().index()
        return self

    @staticmethod
    def get_parser():
        parser = argparse.ArgumentParser(
            parents=[
                wbgit.WBRepo.get_parser(),
                wbgithub.WBGitHub.get_parser(),
                wbredmine.WBRedmine.get_parser(),
            ],
            conflict_handler='resolve',
        )

        parser.add_argument('--dry-run', action='store_const',
                            const=True,
                            help='display actions but do not carry them out')
        return parser

    @staticmethod
    def factory(argv):
        return WBBackportSetRelease(
            WBBackportSetRelease.get_parser().parse_args(argv))

    def run(self):
        self.open()
        for (issue, log) in self.set_release().iteritems():
            if 'ERROR' in log:
                logging.error(log)
            elif 'SKIP' in log:
                logging.debug(log)
            else:
                logging.info(log)

    def set_release(self):
        fixes_re = re.compile(r"Fixes\:? #(\d+)")
        tracker_re = re.compile(self.redmine.args.redmine_url +
                                "/issues/(\d+)")
        pr_re = re.compile("https?://github.com/" +
                           self.github.args.github_repo + "/pull/(\d+)")

        def url_pr(pr):
            return ("https://github.com/" + self.github.args.github_repo +
                    "/pull/" + str(pr['number']))

        kwargs = {
            'limit': 500,
            'tracker_id': self.redmine.tracker2tracker_id['Backport'],
            'status_id': self.redmine.status2status_id['In Progress'],
        }
        r = self.redmine.r

        logs = {}
        for issue in r.issue.filter(**kwargs):
            prs = pr_re.findall(issue['description'])
            if len(prs) != 1:
                logs[issue['id']] = (
                    "ERROR: expected exactly one PR got " + str(prs))
                continue
            pr = self.github.repos().pulls(prs[0]).get()
            issues = fixes_re.findall(pr['body'])
            issues += tracker_re.findall(pr['body'])
            if not pr['merged']:
                logs[issue['id']] = "SKIP: not merged " + url_pr(pr)
                continue
            if str(issue['id']) not in issues:
                logs[issue['id']] = (
                    "ERROR: expected the PR to reference the issue")
                continue
            if pr['base']['ref'] not in util.releases():
                logs[issue['id']] = (
                    "ERROR: " + pr['base']['ref'] +
                    " is not a known stable branch")
                continue
            events = self.github.repos().issues(prs[0]).events().get()
            merge_sha = None
            for event in events:
                if event['event'] == 'merged':
                    merge_sha = event['commit_id']
                    break
            assert merge_sha
            describe = subprocess.check_output(
                "cd " + self.git.args.git_directory + " ;"
                " git describe " + merge_sha, shell=True)
            version = re.findall("^(v\d+\.\d+\.\d+)", describe)
            if not version:
                logs[issue['id']] = (
                    "ERROR: " + describe.strip() +
                    " as returned by git describe " +
                    merge_sha + " is not parseable")
                continue
            ((major, minor),) = re.findall("(.*)\.(.*)", version[0])
            minor = "%d" % (int(minor) + 1)
            version = major + "." + minor
            if version not in self.redmine.version2version_id:
                logs[issue['id']] = (
                    "ERROR: " + str(version) + " obtained from " +
                    describe + " as returned by git describe " +
                    merge_sha + " is not a known version")
                continue
            logs[issue['id']] = (
                "set version " + version +
                " on http://tracker.ceph.com/issues/" + str(issue['id']) +
                " https://github.com/ceph/ceph/pull/" + str(pr['number']))
            if not self.args.dry_run:
                version = self.redmine.version2version_id[version]
                status_id = self.redmine.status2status_id['Resolved']
                r.issue.update(issue['id'],
                               version_id=version,
                               status_id=status_id)
        return logs
